:root {
    --spinner-size: 60px;
}


body {
    margin: 0;
    /* overflow: hidden;  don't show scrolling bars */
}

canvas {
    position: absolute;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;
    z-index: -100;
}

#info {
    position: absolute;
    bottom: 10px;
    text-align: center;
    width: 100%;
    z-index: 100;
    display: block;
    opacity: 0.5;
}

#loading {
    position: absolute;
    bottom: 50%;
    left: 50%;
    z-index: 100;
    display: block;
}

#right_btn_group {
    position: absolute;
    top: 5px;
    right: 10px;
    width: 30px;
    z-index: 100;
    display: block;
    opacity: 0.5;
}

#add_class {
    z-index: 2000;
    display: none;

}

.atom-spinner,
.atom-spinner * {
    box-sizing: border-box;
}

.atom-spinner {
    height: var(--spinner-size);
    width: var(--spinner-size);
    overflow: hidden;
}

.atom-spinner .spinner-inner {
    position: relative;
    display: block;
    height: 100%;
    width: 100%;
}

.atom-spinner .spinner-circle {
    display: block;
    position: absolute;
    color: #ffa500;
    font-size: calc(var(--spinner-size) * 0.24);
    top: 50%;
    left: 50%;
    transform: translate(-50%, -50%);
}

.atom-spinner .spinner-line {
    position: absolute;
    width: 100%;
    height: 100%;
    border-radius: 50%;
    animation-duration: 1s;
    border-left-width: calc(var(--spinner-size) / 25);
    border-top-width: calc(var(--spinner-size) / 25);
    border-left-color: #ffa500;
    border-left-style: solid;
    border-top-style: solid;
    border-top-color: transparent;
}

.atom-spinner .spinner-line:nth-child(1) {
    animation: atom-spinner-animation-1 1s linear infinite;
    transform: rotateZ(120deg) rotateX(66deg) rotateZ(0deg);
}

.atom-spinner .spinner-line:nth-child(2) {
    animation: atom-spinner-animation-2 1s linear infinite;
    transform: rotateZ(240deg) rotateX(66deg) rotateZ(0deg);
}

.atom-spinner .spinner-line:nth-child(3) {
    animation: atom-spinner-animation-3 1s linear infinite;
    transform: rotateZ(360deg) rotateX(66deg) rotateZ(0deg);
}

@keyframes atom-spinner-animation-1 {
    100% {
        transform: rotateZ(120deg) rotateX(66deg) rotateZ(360deg);
    }
}

@keyframes atom-spinner-animation-2 {
    100% {
        transform: rotateZ(240deg) rotateX(66deg) rotateZ(360deg);
    }
}

@keyframes atom-spinner-animation-3 {
    100% {
        transform: rotateZ(360deg) rotateX(66deg) rotateZ(360deg);
    }
}

#progressBar {
    height: 5px;
    position: fixed;
    opacity: 1.0;
    z-index: 100;
    background: #FFA500;
    overflow: hidden;
    animation: progressBar 1.5s ease-in-out;
    animation-iteration-count: infinite;
    animation-fill-mode: both;
    bottom: 0;
    content: "";
    display: block;
    left: 0;
}

#bufferBar {
    height: 5px;
    position: fixed;
    opacity: 1.0;
    z-index: 50;
    background: #b2b2b2;
    overflow: hidden;
    animation: progressBar 1.5s ease-in-out;
    animation-iteration-count: infinite;
    animation-fill-mode: both;
    bottom: 0;
    content: "";
    display: block;
    left: 0;
    visibility: visible;
    width: 0%;
}

#greyOut {
    pointer-events: none;
    display: block;
    position: fixed;
    top: 0;
    bottom: 0;
    left: 0;
    right: 0;
    width: 100%;
    height: 100%;
    background-color: black;
    opacity: 0.05;
    z-index: 1;
    text-align: center;
}

#hamburger-menu {
    position: absolute;
    display: block;
    width: 100%;
    height: 100%;
}

#menu__toggle {
    opacity: 0;
}

#menu__toggle:checked+.menu__btn>span {
    transform: rotate(45deg);
}

#menu__toggle:checked+.menu__btn>span::before {
    top: 0;
    transform: rotate(0deg);
}

#menu__toggle:checked+.menu__btn>span::after {
    top: 0;
    transform: rotate(90deg);
}

#menu__toggle:checked~.menu__box {
    left: 0 !important;
}

.menu__btn {
    position: absolute;
    top: 20px;
    left: 20px;
    width: 26px;
    height: 26px;
    cursor: pointer;
    z-index: 1;
}

.menu__btn>span,
.menu__btn>span::before,
.menu__btn>span::after {
    display: block;
    position: absolute;
    width: 100%;
    height: 2px;
    background-color: #616161;
    transition-duration: .25s;
}

.menu__btn>span::before {
    content: '';
    top: -8px;
}

.menu__btn>span::after {
    content: '';
    top: 8px;
}

.menu__box {
    display: block;
    position: fixed;
    top: 0;
    left: -100%;
    width: 40%;
    height: 100%;
    margin: 0;
    padding: 30px 0;
    /* padding from the top */
    list-style: none;
    background-color: #ECEFF1;
    opacity: 95%;
    box-shadow: 2px 2px 6px rgba(0, 0, 0, .4);
    transition-duration: .25s;
}

.menu__header {
    padding: 12px 24px;
    text-align: center;
}